@@ -5,13 +5,13 @@
 #
 ################################################################################
 #
-#  $Revision: 7 $
+#  $Revision: 13 $
 #  $Author: mhx $
-#  $Date: 2005/10/03 17:02:52 +0200 $
+#  $Date: 2008/01/04 10:47:39 +0100 $
 #
 ################################################################################
 #
-#  Version 3.x, Copyright (C) 2004-2005, Marcus Holland-Moritz.
+#  Version 3.x, Copyright (C) 2004-2008, Marcus Holland-Moritz.
 #  Version 2.x, Copyright (C) 2001, Paul Marquess.
 #  Version 1.x, Copyright (C) 1999, Kenneth Albanowski.
 #
@@ -32,6 +32,12 @@ use Cwd;
 
 # TODO: - extra arguments to Configure
 
+#
+#  --test-archives=1      check if archives can be read
+#  --test-archives=2      like 1, but also extract archives
+#  --test-archives=3      like 2, but also apply patches
+#
+
 my %opt = (
   prefix  => '/tmp/perl/install/<config>/<perl>',
   build   => '/tmp/perl/build/<config>',
@@ -39,6 +45,7 @@ my %opt = (
   force   => 0,
   test    => 0,
   install => 1,
+  'test-archives' => 0,
 );
 
 my %config = (
@@ -91,12 +98,39 @@ my @patch = (
   },
   {
     perl => [
-              qr/^5\.004_0[1234]/,
+              qr/^5\.004_0[1234]$/,
             ],
     subs => [
               [ \&patch_doio ],
             ],
   },
+  {
+    perl => [
+              qw/
+                5.005
+                5.005_01
+                5.005_02
+              /,
+            ],
+    subs => [
+              [ \&patch_sysv, old_format => 1 ],
+            ],
+  },
+  {
+    perl => [
+              qw/
+                5.005_03
+                5.005_04
+              /,
+              qr/^5\.6\.[0-2]$/,
+              qr/^5\.7\.[0-3]$/,
+              qr/^5\.8\.[0-8]$/,
+              qr/^5\.9\.[0-5]$/
+            ],
+    subs => [
+              [ \&patch_sysv ],
+            ],
+  },
 );
 
 my(%perl, @perls);
@@ -110,6 +144,7 @@ GetOptions(\%opt, qw(
   force
   test
   install!
+  test-archives=i
 )) or pod2usage(2);
 
 if (exists $opt{config}) {
@@ -139,6 +174,30 @@ else {
   @perls = sort keys %perl;
 }
 
+if ($opt{'test-archives'}) {
+  my $test = 'test';
+  my $cwd = cwd;
+  -d $test or mkpath($test);
+  chdir $test or die "chdir $test: $!\n";
+  for my $perl (@perls) {
+    eval {
+      my $d = extract_source($perl{$perl});
+      if ($opt{'test-archives'} > 2) {
+        my $cwd2 = cwd;
+        chdir $d or die "chdir $d: $!\n";
+        patch_source($perl{$perl}{version});
+        chdir $cwd2 or die "chdir $cwd2:$!\n"
+      }
+      rmtree($d) if -e $d;
+    };
+    warn $@ if $@;
+  }
+  chdir $cwd or die "chdir $cwd: $!\n";
+  print STDERR "cleaning up\n";
+  rmtree($test);
+  exit 0;
+}
+
 my %current;
 
 for my $cfg (@{$opt{config}}) {
@@ -211,7 +270,8 @@ sub extract_source
 {
   my $perl = shift;
 
-  print "reading $perl->{source}\n";
+  my $what = $opt{'test-archives'} ? 'test' : 'read';
+  print "${what}ing $perl->{source}\n";
 
   my $target;
 
@@ -222,18 +282,20 @@ sub extract_source
     $target = $t;
   }
 
-  if (-d $target) {
-    print "removing old build directory $target\n";
-    rmtree($target);
+  if ($opt{'test-archives'} == 0 || $opt{'test-archives'} > 1) {
+    if (-d $target) {
+      print "removing old build directory $target\n";
+      rmtree($target);
+    }
+
+    print "extracting $perl->{source}\n";
+
+    Archive::Tar->extract_archive($perl->{source})
+        or die "extract failed: " . Archive::Tar->error() . "\n";
+
+    -d $target or die "oooops, $target not found\n";
   }
 
-  print "extracting $perl->{source}\n";
-
-  Archive::Tar->extract_archive($perl->{source})
-      or die "extract failed: " . Archive::Tar->error() . "\n";
-
-  -d $target or die "oooops, $target not found\n";
-
   return $target;
 }
 
@@ -273,13 +335,13 @@ sub build_and_install
 sub patch_db
 {
   my $ver = shift;
-  print "patching DB_File\n";
+  print "patching ext/DB_File/DB_File.xs\n";
   run_or_die("sed -i -e 's/<db.h>/<db$ver\\/db.h>/' ext/DB_File/DB_File.xs");
 }
 
 sub patch_doio
 {
-  patch('doio.c', <<'END');
+  patch(<<'END');
 --- doio.c.org	2004-06-07 23:14:45.000000000 +0200
 +++ doio.c	2003-11-04 08:03:03.000000000 +0100
 @@ -75,6 +75,16 @@
@@ -302,11 +364,52 @@ sub patch_doio
 END
 }
 
+sub patch_sysv
+{
+  my %opt = @_;
+
+  # check if patching is required
+  return if $^O ne 'linux' or -f '/usr/include/asm/page.h';
+
+  if ($opt{old_format}) {
+    patch(<<'END');
+--- ext/IPC/SysV/SysV.xs.org	1998-07-20 10:20:07.000000000 +0200
++++ ext/IPC/SysV/SysV.xs	2007-08-12 10:51:06.000000000 +0200
+@@ -3,9 +3,6 @@
+ #include "XSUB.h"
+ 
+ #include <sys/types.h>
+-#ifdef __linux__
+-#include <asm/page.h>
+-#endif
+ #if defined(HAS_MSG) || defined(HAS_SEM) || defined(HAS_SHM)
+ #include <sys/ipc.h>
+ #ifdef HAS_MSG
+END
+  }
+  else {
+    patch(<<'END');
+--- ext/IPC/SysV/SysV.xs.org	2007-08-11 00:12:46.000000000 +0200
++++ ext/IPC/SysV/SysV.xs	2007-08-11 00:10:51.000000000 +0200
+@@ -3,9 +3,6 @@
+ #include "XSUB.h"
+ 
+ #include <sys/types.h>
+-#ifdef __linux__
+-#   include <asm/page.h>
+-#endif
+ #if defined(HAS_MSG) || defined(HAS_SEM) || defined(HAS_SHM)
+ #ifndef HAS_SEM
+ #   include <sys/ipc.h>
+END
+  }
+}
+
 sub patch
 {
-  my($file, $patch) = @_;
-  print "patching $file\n";
-  my $diff = "$file.diff";
+  my($patch) = @_;
+  print "patching $_\n" for $patch =~ /^\+{3}\s+(\S+)/gm;
+  my $diff = 'tmp.diff';
   write_or_die($diff, $patch);
   run_or_die("patch -s -p0 <$diff");
   unlink $diff or die "unlink $diff: $!\n";
@@ -395,12 +498,12 @@ and don't install them, run:
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004-2005, Marcus Holland-Moritz.
+Copyright (c) 2004-2008, Marcus Holland-Moritz.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-See L<Devel::PPPort>.
+See L<Devel::PPPort> and L<HACKERS>.
 
